/*
 * Copyright 2012          Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#ifndef __DEF_MF_UG_WINSET_H_
#define __DEF_MF_UG_WINSET_H_

#include <stdio.h>
#include <assert.h>
#include <Elementary.h>
#include <glib.h>
#include <vconf.h>
#include <vconf-keys.h>


#include "mf-ug-main.h"
#include "mf-ug-conf.h"
#include "mf-ug-fs-util.h"


/******************Navigation Bar Definition ***********/
#define NAVI_MORE_BUTTON_PART		"toolbar_more_btn"
#define NAVI_BOTTOM_BUTTON_1_PART	"toolbar_button1"
#define NAVI_BOTTOM_BUTTON_2_PART	"toolbar_button2"
#define NAVI_BUTTON_STYLE		"naviframe/toolbar/default"
#define NAVI_BUTTON_EDIT		"naviframe/more/default"

#define TITLE_BTN_STYLE	"elm/button/base/naviframe/title_icon"
#define TITLE_LEFT_BTN		"title_left_btn"
#define TITLE_RIGHT_BTN		"title_right_btn"

#define MF_UG_NAVI_STYLE_ENABLE "basic"
#define	MF_UG_UPPER_HEIGHT	(113*elm_config_scale_get())
#define MF_UG_UPPER_WIDTH	(480*elm_config_scale_get())

Evas_Object *mf_ug_navi_bar_create_navi_bar(Evas_Object *parent);
void mf_ug_navi_bar_set_new_content(Evas_Object *pLayout, Evas_Object *NaviContent);
void mf_ug_navi_bar_create_default_view(void *data);
ugNaviBar *mf_ug_navi_bar_get_in_use_navi(void *data);
void mf_ug_navi_bar_reset_navi_object(ugNaviBar *pNavi_s);
void mf_ug_navi_bar_refresh_navibar_recovered_view(void *data, ugNaviBar *pNaviStruct);
void mf_ug_navi_bar_remove_navi_list_item_by_label(void *data, const char *pNaviLabel);
void mf_ug_navi_bar_set_blocking(void *data, bool bblock_during_transition);
void mf_ug_navi_bar_remove_previous_contents(void *data);
void mf_ug_navi_bar_del_item_directly(void *data);
void mf_ug_navi_bar_create_new_folder_view(void *data);
ugNaviBar *ug_mf_navi_bar_get_navi_from_navilist(Eina_List *navilist, const char *label);
ugNaviBar *mf_ug_navi_bar_get_navi_struct_by_label(void *data, const char *pNaviLabel);
void mf_ug_navi_bar_title_set(void *data);
Evas_Object *mf_ug_navibar_create_notify_popup(void *data,
				    Evas_Object *parent,
				    char *label,
				    bool time_out,
				    void func(void *data, Evas_Object *obj, void *event_info));

/********** Control Bar Definition ********/

#define UG_CTRL_ICON_CANCEL					UG_ICON_PATH"/myfile_icon_cancel.png"
#define UG_CTRL_ICON_SAVE						UG_ICON_PATH"/myfile_icon_done.png"
#define UG_TITLE_ICON_HOME	            			UG_ICON_PATH"/Controlbar/U01_controlbar_cion_home.png"
#define UG_TITLE_ICON_HOME_PRESS	            		UG_ICON_PATH"/Controlbar/U01_controlbar_cion_home_press.png"
#define UG_TITLE_ICON_UPPER	            			UG_ICON_PATH"/Controlbar/U01_controlbar_cion_up_folder.png"
#define UG_TITLE_ICON_UPPER_PRESS	            			UG_ICON_PATH"/Controlbar/U01_controlbar_cion_up_folder_press.png"

typedef enum _eCtrlBarButtonType eCtrlBarButtonType;

enum _eCtrlBarButtonType {
	CTRL_BUTTON_CANCEL = 0,
	CTRL_BUTTON_DONE,
	CTRL_BUTTON_ADD,
	CTRL_BUTTON_OTHER_RIONTONGS,
	CTRL_BUTTON_MUSIC,
	CTRL_BUTTON_MAX
};

void mf_ug_ctrl_bar_set(void *data, ugNaviBar *pNavi_s);
void mf_ug_ctrl_bar_set_item_disable(void *data);


/**********Tab Bar Definition *************/
Evas_Object *mf_ug_tab_bar_create(Evas_Object *parent);
Elm_Object_Item *mf_ug_tab_bar_append_item(Evas_Object *pTabBar, char *pTabIconPath, char *pTabBarLabel, void *user_data);
void mf_ug_tab_bar_set_item_view(Evas_Object *tab_layout, Evas_Object *view);
void mf_ug_tab_bar_remove_item_by_label(Evas_Object *pTabBar, char *pTabLabel);
void mf_ug_tab_bar_remove_all_items(Evas_Object *pTabBar);
void mf_ug_tab_bar_set_selected_item(Elm_Object_Item *pItem);
void mf_ug_tab_bar_set_item_disable(Evas_Object *pTabBar, Eina_Bool disable);
void mf_ug_ctrl_bar_recover(void *data);
Elm_Object_Item *mf_ug_tab_bar_get_item_by_label(Evas_Object *pTabBar, char *pTabLabel);
int mf_ug_tab_bar_items_count_get(Evas_Object *pTabBar);
void mf_ug_navi_bar_button_set_disable(void *data, bool disable);


/***********	Popup Definition	************/
typedef enum _mf_ug_popup_mode mf_ug_popup_mode;
enum _mf_ug_popup_mode {
	UG_POPMODE_MIN = 0,
	UG_POPMODE_TEXT,
	UG_POPMODE_TITLE_TEXT,
	UG_POPMODE_TEXT_TWO_BTN,
	UG_POPMODE_TITLE_TEXT_TWO_BTN,
	UG_POPMODE_TEXT_BTN,
	UG_POPMODE_TITLE_TEXT_BTN,
	UG_POPMODE_TITLE_TEXT_THREE_BTN,
	UG_POPMODE_PROGRESSBAR,
	UG_POPMODE_SEARCH,
	UG_POPMODE_LIST_BY,
	UG_POPMPDE_MAX
};

typedef enum _mf_ug_popup_event_type mf_ug_popup_event_type;
enum _mf_ug_popup_event_type {
	UG_ELM_POPUP_NONE,
	UG_ELM_POPUP_YES,
	UG_ELM_POPUP_NO,
	UG_ELM_POPUP_OK,
	UG_ELM_POPUP_CANCLE,
	UG_ELM_POPUP_AUTO_RENAME,
	UG_ELM_POPUP_REPLACE,
	UG_ELM_POPUP_PHONE,
	UG_ELM_POPUP_MEMORY,
	UG_ELM_POPUP_MAX
};

/***********	Popup API		************/
Evas_Object *mf_ug_popup_create(void *data, mf_ug_popup_mode popupMode, char *title, const char *context, const char *first_btn_text, const char *second_btn_text,
			const char *third_btn_text, Evas_Smart_Cb func, void *param);
void mf_ug_popup_indicator_popup(const char *text);
void mf_ug_context_popup_create_more(void *data, Evas_Object *parent);


/******************* Genlist API      *********************/

void mf_ug_upper_genlist_item_append(Evas_Object *pGenlist, void *data);
void mf_ug_genlist_goto_myfile_item_append(Evas_Object *pGenlist, void *data);
Evas_Object *mf_navi_bar_upper_create(Evas_Object *parent);
void mf_ug_genlist_goto_myfile_style_set(void *data);
void mf_ug_genlist_upper_style_set(void *data);
Evas_Object *mf_ug_genlist_create_content_list_view(void *data);
Evas_Object *__mf_ug_genlist_create_gl(void *data);
Evas_Object *mf_ug_genlist_create_list_new_folder_style(void *data);
Evas_Object *mf_ug_genlist_create_path_info(Evas_Object *parent, Elm_Genlist_Item_Class *itc, char *info);
Elm_Object_Item *mf_ug_genlist_item_append(Evas_Object *parent,
				      ugFsNodeInfo *pNode,
				      void *data,
				      int groudValue,
				      Elm_Genlist_Item_Class *itc);
void mf_ug_genlist_selected_gl(void *data, Evas_Object *obj, void *event_info);

#endif
